@@ -66,7 +66,7 @@ u_char param_select[] = {
    LABEL | OBJINST,				/* P_SCALE */
    POLYGON | SPLINE | ARC | PATH,		/* P_LINEWIDTH */
    ALL_TYPES, 					/* P_COLOR (tbd) */
-   ALL_TYPES,					/* P_EXPRESSION */
+   ALL_TYPES, 					/* P_EXPRESSION */
    POLYGON | SPLINE | LABEL | OBJINST | ARC 	/* P_POSITION */
 };
 
@@ -414,7 +414,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
    int i, j, k, ival;
    char *key;
    float fval;
-   oparamptr ops, ips;
+   oparamptr dps, ops;
    eparamptr epp;
    XPoint *setpt;
    Boolean needrecalc;	/* for arcs and splines */
@@ -429,24 +429,28 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
 
-	 ops = match_param(thisobj, epp->key);
-	 ips = (pinst != NULL) ?  find_param(pinst, epp->key) : ops;
+	 /* Use the parameter from the instance, if available.	*/
+	 /* Otherwise, revert to the type of the object.	*/
+	 /* Normally they will be the same.			*/
 
-	 if (ips != NULL) {
+         ops = match_param(thisobj, epp->key);
+	 dps = (pinst != NULL) ?  find_param(pinst, epp->key) : ops;
+
+	 if (dps != NULL) {
 
 	    /* Get integer and float values.  Promote types if necessary */
 
-	    switch(ips->type) {
+	    switch(dps->type) {
 	       case XC_INT:
-		  ival = ips->parameter.ivalue;
+		  ival = dps->parameter.ivalue;
 		  fval = (float)(ival);
 		  break;
 	       case XC_FLOAT:
-		  fval = ips->parameter.fvalue;
+		  fval = dps->parameter.fvalue;
 		  ival = (int)(fval + 0.5);
 		  break;
 	       case XC_STRING:
-		  promoted = textprint(ips->parameter.string, pinst);
+		  promoted = textprint(dps->parameter.string, pinst);
 		  if (sscanf(promoted, "%g", &fval) == 1)
 		     ival = (int)(fval + 0.5);
 		  else
@@ -454,7 +458,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 		  free(promoted);
 		  break;
 	       case XC_EXPR:
-		  if ((promoted = evaluate_expr(ips)) == NULL) continue;
+		  if ((promoted = evaluate_expr(dps)) == NULL) continue;
 		  if (sscanf(promoted, "%g", &fval) == 1)
 		     ival = (int)(fval + 0.5);
 		  free(promoted);
@@ -487,14 +491,14 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 		  free_instance_param(thisinst, newop);
 	       
 	       /* Create a new instance parameter */
-	       newop = copyparameter((ips != NULL) ? ips : ops);
+	       newop = copyparameter(dps);
 	       newop->next = thisinst->params;
 	       thisinst->params = newop;
 
 	       /* Change the key from the parent to the child */
 	       if (strcmp(ops->key, refop->key)) {
 		  free(newop->key);
-		  newop->key = mystrdup(refop->key);
+		  newop->key = strdup(refop->key);
 	       }
 	       continue;
 	    }
@@ -1042,7 +1046,7 @@ eparamptr copyeparam(eparamptr cepp, genericptr thiselem)
 
    newepp = make_new_eparam(cepp->key);
    if (IS_OBJINST(thiselem) && (cepp->pdata.refkey != NULL))
-      newepp->pdata.refkey = mystrdup(cepp->pdata.refkey);
+      newepp->pdata.refkey = strdup(cepp->pdata.refkey);
    else
       newepp->pdata.pointno = cepp->pdata.pointno;
    return newepp;
@@ -1079,7 +1083,7 @@ oparamptr copyparameter(oparamptr cops)
 	 newops->parameter.string = stringcopy(cops->parameter.string);
 	 break;
       case XC_EXPR:
-	 newops->parameter.expr = mystrdup(cops->parameter.expr);
+	 newops->parameter.expr = strdup(cops->parameter.expr);
 	 break;
       case XC_INT: case XC_FLOAT:
 	 newops->parameter.ivalue = cops->parameter.ivalue;
@@ -1126,20 +1130,26 @@ void copyparams(objinstptr destinst, objinstptr sourceinst)
 void makeexprparam(objectptr refobject, char *key, char *value, int which)
 {
    oparamptr newops;
+   char *newkey;
+
+   /* Check against postscript reserved names */
+
+   newkey = checkpostscriptname(key, NULL, NULL);
 
    /* Ensure that no two parameters have the same name! */
 
-   if (check_param(refobject, key)) {
+   if (check_param(refobject, newkey)) {
       Wprintf("There is already a parameter with that key!");
    }
 
-   newops = make_new_parameter(key);
+   newops = make_new_parameter(newkey);
    newops->next = refobject->params;
    refobject->params = newops;
    newops->type = XC_EXPR;		/* expression requiring evaluation */
    newops->which = which;
-   newops->parameter.expr = mystrdup(value);
+   newops->parameter.expr = strdup(value);
    incr_changes(refobject);
+   free(newkey);
 }
 
 /*------------------------------------------------------------------*/
@@ -1149,10 +1159,15 @@ void makeexprparam(objectptr refobject, char *key, char *value, int which)
 void makefloatparam(objectptr refobject, char *key, float value)
 {
    oparamptr newops;
+   char *newkey;
+
+   /* Check against postscript reserved names */
+
+   newkey = checkpostscriptname(key, NULL, NULL);
 
    /* Ensure that no two parameters have the same name! */
 
-   if (check_param(refobject, key)) {
+   if (check_param(refobject, newkey)) {
       Wprintf("There is already a parameter with that key!");
    }
 
@@ -1163,6 +1178,7 @@ void makefloatparam(objectptr refobject, char *key, float value)
    newops->which = P_NUMERIC;
    newops->parameter.fvalue = value;
    incr_changes(refobject);
+   free(newkey);
 }
 
 /*----------------------------------------------------------------*/
@@ -1172,20 +1188,26 @@ void makefloatparam(objectptr refobject, char *key, float value)
 void makestringparam(objectptr refobject, char *key, stringpart *strptr)
 {
    oparamptr newops;
+   char *newkey;
+
+   /* Check against postscript reserved names */
+
+   newkey = checkpostscriptname(key, NULL, NULL);
 
    /* Ensure that no two parameters have the same name! */
 
-   if (check_param(refobject, key)) {
+   if (check_param(refobject, newkey)) {
       Wprintf("There is already a parameter with that key!");
    }
 
-   newops = make_new_parameter(key);
+   newops = make_new_parameter(newkey);
    newops->next = refobject->params;
    refobject->params = newops;
    newops->type = XC_STRING;
    newops->which = P_SUBSTRING;
    newops->parameter.string = strptr;
    incr_changes(refobject);
+   free(newkey);
 }
 
 /*--------------------------------------------------------------*/
@@ -1241,7 +1263,7 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
    /* Ensure that no two parameters have the same name! */
 
    if (key) {
-      keyptr = key;
+      keyptr = checkpostscriptname(key, NULL, NULL);
    }
    else {
       strcpy(new_key, param_keys[mode]);
@@ -1277,7 +1299,7 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 	 free_element_param(*gelem, epp);
 	 Fprintf(stderr, "Error: Attempt to link a parameter to "
 			"a parameter of a different type\n");
-	 return;
+	 goto param_done;
       }
       if ((newops = match_instance_param(areastruct.topinstance, keyptr)) == NULL) {
          newops = make_new_parameter(keyptr);
@@ -1293,7 +1315,7 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 	 /* should redraw.						*/
          drawarea(NULL, NULL, NULL);
          newops = NULL;
-	 return;
+	 goto param_done;
       }
    }
 
@@ -1422,6 +1444,9 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 	    break;
       }
    }
+
+param_done:
+   if (keyptr != new_key) free(keyptr);
 }
 
 /*--------------------------------------------------------------*/
@@ -1573,6 +1598,7 @@ void makeparam(labelptr thislabel, char *key)
 {
    oparamptr newops;
    stringpart *begpart, *endpart;
+   char *newkey;
 
    /* cannot form a parameter on a top-level page */
 
@@ -1589,6 +1615,10 @@ void makeparam(labelptr thislabel, char *key)
       return;
    }
 
+   /* check parameter against reserved PostScript names */
+
+   newkey = checkpostscriptname(key, NULL, NULL);
+
    /* First, place PARAM_START and PARAM_END structures at the	*/
    /* intended parameter boundaries				*/
 
@@ -1615,13 +1645,13 @@ void makeparam(labelptr thislabel, char *key)
       endpart = makesegment(&thislabel->string, NULL);
    }
    begpart->type = PARAM_START;
-   begpart->data.string = (char *)malloc(1 + strlen(key));
-   strcpy(begpart->data.string, key);
+   begpart->data.string = (char *)malloc(1 + strlen(newkey));
+   strcpy(begpart->data.string, newkey);
    endpart->type = PARAM_END;
 
    /* Now move the sections of string to the object parameter */
 
-   newops = make_new_parameter(key);
+   newops = make_new_parameter(newkey);
    newops->next = topobject->params;
    topobject->params = newops;
    newops->type = XC_STRING;
@@ -1632,6 +1662,7 @@ void makeparam(labelptr thislabel, char *key)
 
    textend = 0;
    incr_changes(topobject);
+   free(newkey);
 }
 
 /*--------------------------------------------------------------*/
