@@ -255,6 +255,7 @@ objectptr *new_library_object(short, char *, objlistptr *);
 Boolean library_object_unique(short, objectptr, objlistptr);
 void add_object_to_library(short, objectptr);
 u_char *find_delimiter(u_char *);
+char standard_delimiter_end(char);
 
 #ifdef TCL_WRAPPER
 void setfile(char *, int);
@@ -537,42 +538,43 @@ extern Tcl_Obj *tcltoplevel(objinstptr);
 extern Boolean RemoveFromNetlist(objectptr, genericptr);
 extern labelptr NetToLabel(int, objectptr);
 extern void NameToPosition(objinstptr, labelptr, XPoint *);
+extern XPoint *NetToPosition(int, objectptr);
 extern void invalidate_netlist(objectptr);
 extern void remove_netlist_element(objectptr, genericptr);
 extern int updatenets(objinstptr);
 extern void createnets(objinstptr);
 extern void destroynets();
-extern void gencalls(objectptr);
 extern Boolean nonnetwork(polyptr);
 extern int globalmax();
-extern void gennetlist(objectptr, objinstptr);
-extern void gencalllist(objectptr);
+extern void gennetlist(objinstptr);
+extern void gencalls(objectptr);
 extern void search_on_siblings(objinstptr, objinstptr, pushlistptr,
 		short, short, short, short);
-extern int is_resolved(genericptr *, pushlistptr, objectptr *);
+extern void copy_bus(Genericlist *, Genericlist *);
+extern Genericlist *is_resolved(genericptr *, pushlistptr, objectptr *);
 extern Boolean highlightnet(objectptr, objinstptr, int, u_char);
 extern int pushnetwork(pushlistptr, objectptr);
 extern int onsegment(XPoint *, XPoint *, XPoint *);
 extern Boolean neardist(long);
 extern Boolean nearpoint(XPoint *, XPoint *);
-extern int make_tmp_pin(objectptr, XPoint *, objinstptr, int);
 extern int searchconnect(XPoint *, int, objinstptr, int);
-extern int translateup(int, objectptr, objectptr, objinstptr);
-extern void addpoly(objectptr, polyptr, int, int);
+extern Genericlist *translateup(Genericlist *, objectptr, objectptr, objinstptr);
+extern Genericlist *addpoly(objectptr, polyptr, Genericlist *);
 extern long zsign(long, long);
-extern void mergenets(objectptr, int, int, int);
+extern Boolean mergenets(objectptr, Genericlist *, Genericlist *);
 extern void removecall(objectptr, CalllistPtr);
-extern void addpin(objectptr, labelptr, int, int);
+extern Genericlist *addpin(objectptr, objinstptr, labelptr, Genericlist *);
 extern void addcall(objectptr, objectptr, objinstptr);
-extern Boolean addport(objectptr, int);
-extern void addportcall(objectptr, objinstptr, objectptr, int, int);
+extern void addport(objectptr, Genericlist *);
+extern Boolean addportcall(objectptr, Genericlist *, Genericlist *);
 extern void makelocalpins(objectptr, CalllistPtr, char *);
 extern int porttonet(objectptr, int);
-extern int pointtonet(objectptr, XPoint *, int, int, objinstptr);
-extern int *pointtolist(objectptr, XPoint *, int *);
 extern stringpart *nettopin(int, objectptr, char *);
-extern int pintonet(objectptr, objinstptr, labelptr, int);
-extern linkedNet *nametonet(objectptr, objinstptr, char *);
+extern Genericlist *pointtonet(objectptr, objinstptr, XPoint *);
+extern Genericlist *pintonet(objectptr, objinstptr, labelptr);
+extern Genericlist *nametonet(objectptr, objinstptr, char *);
+extern Genericlist *new_tmp_pin(objectptr, XPoint *, char *, char *, Genericlist *);
+extern Genericlist *make_tmp_pin(objectptr, objinstptr, XPoint *, Genericlist *);
 extern char *parseinfo(objectptr, CalllistPtr, char *, char *, Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
 extern void writeflat(objectptr, CalllistPtr, char *, FILE *, char *);
@@ -582,6 +584,7 @@ extern void writenet(objectptr, char *, char *);
 extern void writepcb(struct Ptab **, objectptr, CalllistPtr, char *, char *);
 extern void outputpcb(struct Ptab *, FILE *);
 extern void freepcb(struct Ptab *);
+extern void freegenlist(Genericlist *);
 extern void freepolylist(PolylistPtr *);
 extern void freelabellist(LabellistPtr *);
 extern void freeports(CalllistPtr);
@@ -592,7 +595,6 @@ extern void destroynets(objectptr);
 extern int  cleartraversed(objectptr);
 extern int  checkvalid(objectptr);
 extern void clearlocalpins(objectptr);
-extern void freelinkednet(linkedNet **);
 
 /* from parameter.c: */ 
 
@@ -688,7 +690,7 @@ extern objectptr NameToObject(char *, objinstptr *, Boolean);
 extern void callwritenet(xcWidget, pointertype, caddr_t);
 extern void doxschema(xcWidget, pointertype, caddr_t);
 extern void startconnect(xcWidget, caddr_t, caddr_t);
-extern int connectivity(xcWidget, caddr_t, caddr_t);
+extern void connectivity(xcWidget, caddr_t, caddr_t);
 extern Boolean setobjecttype(objectptr);
 extern void pinconvert(labelptr, pointertype);
 extern void dopintype(xcWidget, pointertype, caddr_t);
@@ -749,6 +751,7 @@ extern void charprint(char *, stringpart *, int);
 extern char *stringprint(stringpart *, objinstptr);
 extern char *textprint(stringpart *, objinstptr);
 extern char *textprintsubnet(stringpart *, objinstptr, int);
+extern char *textprintnet(char *, char *, Genericlist *);
 extern int textcomp(stringpart *, char *, objinstptr);
 extern int textncomp(stringpart *, char *, objinstptr);
 extern int stringcomp(stringpart *, stringpart *);
@@ -759,7 +762,7 @@ extern stringpart *stringcopy(stringpart *);
 extern stringpart *stringcopyall(stringpart *, objinstptr);
 extern stringpart *stringcopyback(stringpart *, objinstptr);
 extern stringpart *deletestring(stringpart *, stringpart **, objinstptr);
-extern int *break_up_bus(labelptr, objinstptr, int *);
+extern Genericlist *break_up_bus(labelptr, objinstptr, Genericlist *);
 extern int sub_bus_idx(labelptr, objinstptr);
 extern Boolean pin_is_bus(labelptr, objinstptr);
 extern int find_cardinal(int, labelptr, objinstptr);
