@@ -221,7 +221,7 @@ stringpart *nextstringpart(stringpart *strptr, objinstptr thisinst)
 
 stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisinst)
 {
-   stringpart *strptr, *nextptr;
+   stringpart *strptr = NULL, *nextptr;
    char *key;
    oparamptr ops;
 
@@ -268,7 +268,8 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
    free(dstr);
 
    /* attempt to merge, if legal */
-   mergestring(strptr);
+   if (strptr)
+      mergestring(strptr);
 
    return strptr;
 }
@@ -549,6 +550,53 @@ char *textprintsubnet(stringpart *strtop, objinstptr localinst, int subnet)
 	    }
 	 }
       }
+      else {
+	 /* Promote a non-bus label to a bus label */
+	 substr = malloc(10 + strlen(newstr));
+	 strcpy(substr, newstr);
+	 endptr = substr;
+	 while ((*endptr) != '\0') endptr++;
+	 sprintf(endptr, "%c%d%c", areastruct.buschar, subnet,
+		standard_delimiter_end(areastruct.buschar));
+	 free(newstr);
+	 return substr;
+      }
+   }
+   return newstr;
+}
+
+/*----------------------------------------------------------------------*/
+/* Another variant:  Print a subnet list according to the entries in	*/
+/* a netlist (Genericlist *) pointer.  This includes the condition in	*/
+/* which the list is a single wire, not a bus.  If "pinstring" is non-	*/
+/* NULL, it will be used as the name of the subnet.  Otherwise, 	*/
+/* "prefix" will be used, and will be appended with the net ID of the	*/
+/* first net in the sublist to make the identifier unique.		*/
+/*----------------------------------------------------------------------*/
+
+char *textprintnet(char *prefix, char *pinstring, Genericlist *sublist)
+{
+   char *newstr, *sptr;
+   buslist *sbus;
+   int i;
+
+   if (sublist->subnets == 0) {
+      newstr = (char *)malloc(strlen(prefix) + 10); 
+      sprintf(newstr, "%s%d", prefix, sublist->net.id);
+   }
+   else {	/* just make a comma-separated list */
+      newstr = (char *)malloc(strlen(prefix) + 20 + 3 * sublist->subnets);
+      sbus = sublist->net.list;
+      sprintf(newstr, "%s%d%c", prefix, sbus->netid, areastruct.buschar);
+      for (i = 0; i < sublist->subnets; i++) {
+	 sbus = sublist->net.list + i;
+	 sptr = newstr + strlen(newstr);
+	 if (i != 0)
+	    strcat(sptr++, ",");
+	 sprintf(sptr, "%d", sbus->subnetid);
+      }
+      sptr = newstr + strlen(newstr);
+      sprintf(sptr, "%c", standard_delimiter_end(areastruct.buschar));
    }
    return newstr;
 }
@@ -725,8 +773,11 @@ Boolean pin_is_bus(labelptr thislab, objinstptr thisinst)
 
 /*----------------------------------------------------------------------*/
 /* When encountering a label with bus notation, create a list of	*/
-/* subnets belonging to the bus.  Return the list and set numsubnets	*/
-/* to the size of the list (number of subnets).				*/
+/* subnets belonging to the bus.  Return the list as a pointer to a	*/
+/* Genericlist structure.  This structure is statically allocated and	*/
+/* is expected to have its contents copied into the target netlist	*/
+/* element.								*/
+/*									*/
 /* Unlike the above routine, this routine prints the original string	*/
 /* into a char* array using textprint() so that escape sequences are	*/
 /* removed and will not affect the result.				*/
@@ -735,18 +786,29 @@ Boolean pin_is_bus(labelptr thislab, objinstptr thisinst)
 /* pin_is_bus() to determine if the label does indeed represent a bus.	*/
 /* break_up_bus() is much slower in determining this.  If break_up_bus	*/
 /* is passed a string that cannot be identified as a bus, then it	*/
-/* returns a NULL list.							*/
+/* returns a NULL pointer.						*/
+/*									*/
+/* If netlist points to a structure with no subnets, then its net ID	*/
+/* is the starting point for the nets returned by break_up_bus.  	*/
+/* Otherwise, netlist is assumed to be a valid netlist for a bus that	*/
+/* matches "blab", and we will use net IDs from this list.		*/
 /*----------------------------------------------------------------------*/
 
-int *break_up_bus(labelptr blab, objinstptr thisinst, int *numsubnets)
+Genericlist *break_up_bus(labelptr blab, objinstptr thisinst, Genericlist *netlist)
 {
-   int *sublist = NULL;
+   static Genericlist *subnets = NULL;
    char *tptr;
-   int bpos, istart, iend, tlen, i;
+   buslist *sbus, *jbus;
+   int bpos, istart, iend, tlen, i, j, netstart, matched;
    char *buspos, *busend, *busptr;
-   int subnetcount = 0;
 
    if (pin_is_bus(blab, thisinst) == FALSE) return NULL;
+   if (subnets == NULL) {
+      /* This happens on the first pass only */
+      subnets = (Genericlist *)malloc(sizeof(Genericlist));
+      subnets->net.list = (buslist *)malloc(sizeof(buslist));
+   }
+   subnets->subnets = 0;
 
    tptr = textprint(blab->string, thisinst);
    tlen = strlen(tptr) + 1;
@@ -762,6 +824,8 @@ int *break_up_bus(labelptr blab, objinstptr thisinst, int *numsubnets)
       goto doneBus;
    }
 
+   netstart = (netlist->subnets == 0) ? netlist->net.id : 0;
+
    bpos = (int)(buspos - tptr);
    busend = find_delimiter(buspos);
 
@@ -772,6 +836,7 @@ int *break_up_bus(labelptr blab, objinstptr thisinst, int *numsubnets)
 
    /* Find the range of each bus */
 
+   matched = 0;
    istart = -1;
    for (busptr = buspos + 1; busptr < busend; busptr++) {
       if (sscanf(busptr, "%d", &iend) == 0) break;
@@ -787,13 +852,31 @@ int *break_up_bus(labelptr blab, objinstptr thisinst, int *numsubnets)
 
 	    /* Create a new list entry for this subnet number */
 
-	    subnetcount++;
-	    if (sublist == NULL)
-	       sublist = (int *)malloc(sizeof(int));
-	    else
-	       sublist = (int *)realloc(sublist, subnetcount * sizeof(int));
+	    subnets->subnets++;
+	    subnets->net.list = (buslist *)realloc(subnets->net.list,
+			subnets->subnets * sizeof(buslist));
 
-	    *(sublist + subnetcount - 1) = i;
+	    sbus = subnets->net.list + subnets->subnets - 1;
+	    sbus->subnetid = i;
+	    if (netstart > 0) {
+	       sbus->netid = netstart++;
+	       matched++;
+	    }
+	    else {
+	       /* Net ID is the net ID for the matching subnet of netlist */
+	       for (j = 0; j < netlist->subnets; j++) {
+		  jbus = netlist->net.list + j;
+		  if (jbus->subnetid == i) {
+		     matched++;
+		     sbus->netid = jbus->netid;
+		     break;
+		  }
+	       }
+	       /* Insert a net ID of zero if it can't be found */
+	       if (j == netlist->subnets) {
+		  sbus->netid = 0;
+	       }
+	    }
 
 	    if (i == iend) break;
 	    else if (istart > iend) i--;
@@ -805,8 +888,7 @@ int *break_up_bus(labelptr blab, objinstptr thisinst, int *numsubnets)
 
 doneBus:
    free(tptr);
-   *numsubnets = subnetcount;
-   return sublist;
+   return (matched == 0) ? NULL : subnets;
 }
 
 /*----------------------------------------------------------------------*/
