@@ -92,9 +92,8 @@ char	 _STR2[250];  /* Specifically for text returned from the popup prompt */
 char	 _STR[150];          /* Generic multipurpose string */
 short	 popups;	             /* total number of popup widgets on the screen */
 
-xcWidget	 message1, message2, message3, top, toolbar, overlay = NULL;
-
 #ifndef TCL_WRAPPER
+xcWidget   message1, message2, message3, top, toolbar, overlay = NULL;
 xcWidget   menuwidgets[MaxMenuWidgets];
 #endif
 
@@ -131,10 +130,11 @@ short beeper;
 short fontcount;
 fontinfo *fonts;
 
-extern short eventmode, textpos, help_up;
+extern short eventmode, textpos;
 extern float version;
 
 #ifndef TCL_WRAPPER
+extern short help_up;
 extern menustruct TopButtons[];
 #ifdef HAVE_XPM
 extern toolbarstruct ToolBar[];
@@ -1625,36 +1625,10 @@ xcTimeOutProc clrmessage(caddr_t clientdata, xcIntervalId *id)
    }
 }
 
+#ifndef TCL_WRAPPER
+
 /*------------------------------------------------------------------------------*/
 
-#ifdef TCL_WRAPPER
-
-/* These should be replaced with script versions */
-
-void W0printf(char *string, char *window)
-{
-    char _STR3[300];
-    sprintf(_STR3, ".xcircuit.%s config -text {%s}", window, string);
-    Tcl_Eval(xcinterp, _STR3);
-}
-
-void W1printf(char *string)
-{
-    W0printf(string, "menubar.message");
-}
-
-void W2printf(char *string)
-{
-    W0printf(string, "infobar.message1");
-}
-
-void Wprintf(char *string)
-{
-    W0printf(string, "infobar.message2");
-}
-
-#else
-
 void W1printf(char *string)
 {
    Arg	wargs[1];
@@ -1843,6 +1817,21 @@ void getcommand(xcWidget cmdw, caddr_t clientdata, caddr_t calldata)
    XtUnmanageChild(cmdw);
 }
 
+#endif /* !TCL_WRAPPER */
+
+#ifdef TCL_WRAPPER
+
+/*------------------------------------------------------------------------------*/
+/* "docommand" de-iconifies the TCL console.					*/
+/*------------------------------------------------------------------------------*/
+
+void docommand()
+{
+   Tcl_Eval(xcinterp, "catch xcircuit::raiseconsole");
+}
+
+#else  /* !TCL_WRAPPER */
+
 /*------------------------------------------------------------------------------*/
 /* "docommand" overlays the message3 widget temporarily with a TextEdit widget. */
 /*------------------------------------------------------------------------------*/
@@ -2316,8 +2305,8 @@ void pre_initialize()
    initmem(areastruct.editstack);
    areastruct.pinpointon = False;
    areastruct.schemon = False;
+   areastruct.buschar = '(';	/* Vector notation for buses */
 
-   /* xobjs.numlibs = LIBS - LIBRARY; */
    xobjs.numlibs = LIBS - LIBRARY - 1;
    xobjs.fontlib.number = 0;
    xobjs.userlibs = (Library *) malloc(xobjs.numlibs * sizeof(Library));
