@@ -32,6 +32,9 @@ typedef unsigned long	u_long;
   #define calloc(a,b) Tcl_Alloc(a * b)
   #define free(a) Tcl_Free((char *)(a))
   #define realloc(a,b) Tcl_Realloc((char *)(a), b)
+  #undef strdup
+  #define strdup Tcl_Strdup
+  extern char *Tcl_Strdup(const char *);
 #endif
 
 /*----------------------------------------------------------------------*/
@@ -689,27 +692,29 @@ typedef struct {
    int subnetid;
 } buslist;
 
-/* Linked array of nets returned by pintonet() and pointtonet(), to	*/
-/* deal with buses by translating them into multiple networks.		*/
+/* Structure mimicking the top part of a Polylist or Labellist	*/
+/* when we just want the netlist information and don't care	*/
+/* which one we're looking at.					*/
 
-typedef struct _linkedNet *linkedNetPtr;
-typedef struct _linkedNet {
-   int netid;
-   int subnetid;
-   linkedNetPtr next;
-} linkedNet;
+typedef struct {
+   union {
+      int id;
+      buslist *list;
+   } net;
+   int subnets;
+} Genericlist; 
 
 /* Linked polygon list */
 
 typedef struct _Polylist *PolylistPtr;
 typedef struct _Polylist
 {
-   objectptr cschem;	/* Schematic containing the polygon	*/
    union {
       int  id;		/* A single net ID, if subnets == 0	*/
       buslist *list;	/* List of net and subnet IDs for a bus	*/
    } net;
    int subnets;		/* Number of subnets; 0 if no subnets	*/
+   objectptr cschem;	/* Schematic containing the polygon	*/
    polyptr poly;
    PolylistPtr next;	/* Next polygon in the linked list	*/
 } Polylist;
@@ -719,12 +724,13 @@ typedef struct _Polylist
 typedef struct _Labellist *LabellistPtr;
 typedef struct _Labellist
 {
-   objectptr cschem;	/* Schematic containing the label	*/
    union {
       int  id;		/* A single net ID, if subnets == 0	*/
       buslist *list;	/* List of net and subnet IDs for a bus	*/
    } net;
    int subnets;		/* Number of subnets; 0 if no subnets	*/
+   objectptr cschem;	/* Schematic containing the label	*/
+   objinstptr cinst;	/* Specific label instance, if applicable */
    labelptr label;
    LabellistPtr next;
 } Labellist;
@@ -755,7 +761,7 @@ typedef struct _Portlist
 typedef struct _Calllist *CalllistPtr;
 typedef struct _Calllist
 {
-   objectptr   cschem;		/* Schematic page containing the instance */
+   objectptr   cschem;		/* Schematic containing the instance called */
    objinstptr  callinst;	/* Instance called */
    objectptr   callobj;		/* Object of instance called */
    int         devindex;	/* if non-zero, index of device in netlist */
@@ -788,7 +794,7 @@ struct Ptab {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   int		netid;
+   Genericlist	*netlist;
    objinstptr	thisinst;
 } Highlight;
 
